UNIT DPMIProc;

INTERFACE

uses WinProcs;

type
  RMode_Regs = record
    EDI:      longint;
    ESI:      longint;
    EBP:      longint;
    Reserved: longint;
    EBX:      longint;
    EDX:      longint;
    ECX:      longint;
    EAX:      longint;
    Flags:    word;
    ES:       word;
    DS:       word;
    FS:       word;
    GS:       word;
    IP:       word;
    CS:       word;
    SP:       word;
    SS:       word;
  end;

function  DPMIPresent: boolean;

procedure DPMIVersion(var vermajor, verminor, flags, CPU: word);

function  DPMI_RMode_Get_Intr_Vector(IntNo: word; var Para, Ofst: word): boolean;

function  DPMI_RMode_Intr(Intno, Flags, CopyWords: word;  var rmode_call: RMode_Regs): boolean;

function  DosAllocRealSegOK(bytes: longint; var para, sel: word): boolean;

function  DosFreeRealSegOK(sel: word): boolean;

IMPLEMENTATION

function DPMIPresent: boolean;

begin
  asm
    mov ax,1686H
    int 2fH
    not ax
  end;
end;

procedure DPMIVersion(var vermajor, verminor, flags, CPU: word);

var
  maj, min, prc:  byte;
  flgs:           word;

begin
  asm
    mov ax,0400h
    int 31h
    mov maj,ah
    mov min,al
    mov flgs,bx
    mov prc,cl
  end;
  vermajor := maj;
  verminor := min;
  flags    := flgs;
  CPU      := 86 + prc*100;
end;

function DPMI_RMode_Get_Intr_Vector(IntNo: word; var Para, Ofst: word): boolean;

var
  pp, oo: word;

begin
  asm
    mov ax,0200h
    mov bx,word ptr IntNo
    int 31h
    jc  @1
    mov pp,cx
    mov oo,dx
    mov ax,1
    jmp @2
    @1:
    xor ax,ax
    @2:
  end;
  Para := pp;
  Ofst := oo;
end;

function DPMI_RMode_Intr(Intno, Flags, CopyWords: word;  var rmode_call: RMode_Regs): boolean;

begin
  If Flags <> 0 then IntNo := IntNo or $100;
    asm
      push di
      mov  ax,0300h
      mov  bx,word ptr intno
      mov  cx,word ptr copywords
      les  di,dword ptr rmode_call
      int  31h
      jc   @1
      mov  ax,1
      jmp  @2
      @1:
      xor  ax,ax;
      @2:
      pop  di
    end;
end;


function DosAllocRealSegOK(bytes: longint; var para, sel: word): boolean;

var
  dw: longint;

begin
  DosAllocRealSegOK := false;
  dw := GlobalDosAlloc(bytes);
  if dw <> 0 then
    begin
      DosAllocRealSegOK := true;
      para := HiWord(dw);
      sel := LoWord(dw);
    end;
end;

function DosFreeRealSegOK(sel: word): boolean;

begin
  DosFreeRealSegOK := (GlobalDosFree(sel) = 0);
end;

begin
end.
{-------------------------------- END OF DPMIPROC --------------------------}

{-------------------------------- START OF BTRIEVE INTERFACE ---------------}

UNIT TURXBTRV;
{$V-}
{$R-}
{                                                                             }
{  Module Name: TURXBTRV.PAS                                                  }
{                                                                             }
{  Description: This is the Btrieve interface for Turbo Pascal (MS-DOS)       }
{               that was published as part of Version 4.  It has been modified}
{               to run in either a Windows environment or under DOS.          }
{                                                                             }
{  Synopsis:    STAT := BTRV (OP, POS.START, DATA.START, DATALEN,             }
{                                KBUF.START, KEY);                            }
{                             where                                           }
{                       OP is an integer,                                     }
{                       POS is a 128 byte array,                              }
{                       DATA is an untyped parameter for the data buffer,     }
{                       DATALEN is the integer length of the data buffer,     }
{                       KBUF is the untyped parameter for the key buffer,     }
{                   and KEY is an integer.                                    }
{                                                                             }
{  Returns:     Btrieve status code (see Appendix B of the Btrieve Manual).   }
{                                                                             }

INTERFACE


{$IFDEF WINDOWS}
USES DPMIProc, WinProcs, WinTypes;
{$ENDIF}
{$IFNDEF WINDOWS}
USES DOS;
{$ENDIF}

const                  { Constants for use by Programs using this Unit }
  B_Open        =  0;
  B_Close       =  1;
  B_Insert      =  2;
  B_Update      =  3;
  B_Delete      =  4;
  B_Get_EQ      =  5;
  B_Get_Next    =  6;
  B_Get_Prev    =  7;
  B_Get_GT      =  8;
  B_Get_GE      =  9;
  B_Get_LT      = 10;
  B_Get_LE      = 11;
  B_Get_Low     = 12;
  B_Get_High    = 13;
  B_Create      = 14;
  B_Stat        = 15;
  B_Extend      = 16;
  B_Set_Dir     = 17;
  B_Get_Dir     = 18;
  B_Begin       = 19;
  B_End         = 20;
  B_Abort       = 21;
  B_Get_Pos     = 22;
  B_Get_Direct  = 23;
  B_Step_Direct = 24;
  B_Stop        = 25;
  B_Version     = 26;
  B_Unlock      = 27;
  B_Reset       = 28;
  B_Set_Owner   = 29;
  B_Clear_Owner = 30;

  MaxKeysUsed   = 2; {Maximum Number of Keys Used. Modify as Req'd}

type

  KEY_SPEC = record
         KEY_POS   : integer;
         KEY_LEN   : integer;
         KEY_FLAGS : integer;
         NOT_USED  : string[4];
         RESERVED  : string[6];
       end;

  FILE_SPEC = record
    case integer of
      1:(REC_LEN   : integer;
         PAGE_SIZE : integer;
         NDX_CNT   : integer;
         NOT_USED  : string[4];
         VARIABLE  : integer;
         RESERVED  : string[4];
         KEY_BUF   : array[0..MaxKeysUsed-1] of KEY_SPEC);
      2:(SPEC_BUF  : string[16 + MaxKeysUsed*sizeof(KEY_SPEC)]);
    end;

function BTRV (OP:integer; var POS,DATA; var DATALEN: integer;
               var KBUF; KEY: integer): integer;

IMPLEMENTATION

function BTRV (OP:integer; var POS,DATA; var DATALEN: integer;
               var KBUF; KEY: integer): integer;
const
     PASCAL_ID = $AAAA;                                    {Pascal language id}
     VAR_ID    = $6176;                 {id for variable length records - 'va'}
     BTR_INT   = $7B;

{  ProcId is used for communicating with the Multi Tasking Version of         }
{  Btrieve. It contains the process id returned from BMulti and should        }
{  not be changed once it has been set.                                       }
{                                                                             }
     ProcId: integer = 0;                       { initialize to no process id }
     MULTI: boolean = false;                { set to true if BMulti is loaded }
     VSet: boolean = false;       { set to true if we have checked for BMulti }

type
     ADDR32 = record                                           {32 bit address}
        OFFSET: integer;
        SEGMENT: integer;
     end;

     BTR_PARMS = record
        USER_BUF_ADDR: ADDR32;                            {data buffer address}
        USER_BUF_LEN: integer;                             {data buffer length}
        USER_CUR_ADDR: ADDR32;                         {currency block address}
        USER_FCB_ADDR: ADDR32;                     {file control block address}
        USER_FUNCTION: integer;                             {Btrieve operation}
        USER_KEY_ADDR: ADDR32;                             {key buffer address}
        USER_KEY_LENGTH: BYTE;                              {key buffer length}
        USER_KEY_NUMBER: BYTE;                                     {key number}
        USER_STAT_ADDR: ADDR32;                         {return status address}
        XFACE_ID: integer;                              {language interface id}
     end;

{$IFDEF WINDOWS}
     Registers = record
       case integer of
          0: (AX,BX,CX,DX,BP,SI,DI,DS,ES,fLAGS: word);
          1: (AL,AH,BL,BH,CL,CH,DL,DH: byte);
       end;
{$ENDIF}

var
     STAT: integer;              {Btrieve status code}
     XDATA: BTR_PARMS;           {Btrieve parameter block}
     REGS: Registers;            {register structure used on interrrupt call}
     Typ: integer;               {type of request made to BMulti}
     i:   integer;               {loop counter}
     tbuf: array[1..5] of char;  {temporary buffer}

{$IFDEF WINDOWS}
     Paragraph : word;
     Offset    : word;

  Procedure INTR(IntNo: word; var Regs: Registers);

  var
    DPMIRegs  : RMode_Regs;
    LowMemAmt : word;
    Para      : word;
    Sel       : word;
    RBuf      : ^BTR_PARMS;
    BXReg     : word;
    AXReg     : word;
    DSReg     : word;
    DXReg     : word;


  begin
    if (GetWinFlags and wf_PMode) = 0 then {If Real Mode then Invoke Interrupt Directly}
      begin
        AXReg := Regs.AX;
        BXReg := Regs.BX;
        DXReg := Regs.DX;
        DSReg := Regs.DS;
        asm
          push ds
          mov BX,BXReg
          mov AX,AXReg
          mov DX,DXReg
          mov DS,DSReg
          int BTR_INT
          pop DS
        end;
        exit;
      end;
    LowMemAmt := sizeof(BTR_PARMS) + sizeof(STAT) + 128 +  {128 is PosBlk            }
                 255 + DATALEN;                            {Maximum Key Length is 255}
    If not DosAllocRealSegOK(LowMemAmt, Para, Sel) then
      begin
        STAT := 9998;  { Could not Allocate Low Memory }
        exit;
      end;
    RBuf := Ptr(Sel,0);
    With RBuf^ do
      begin
        USER_BUF_ADDR.OFFSET  := sizeof(BTR_PARMS) + sizeof(STAT)
                                 + 128 + 255 - 1;                  {data buffer address}
        USER_BUF_ADDR.SEGMENT := Para;
        USER_BUF_LEN          := DATALEN;                          {data buffer length}
        USER_FCB_ADDR.OFFSET  := sizeof(BTR_PARMS)+sizeof(STAT)-1; {file control block address}
        USER_FCB_ADDR.SEGMENT := Para;
        USER_CUR_ADDR.OFFSET  := RBuf^.USER_FCB_ADDR.OFFSET+38;    {currency block address}
        USER_CUR_ADDR.SEGMENT := Para;
        USER_FUNCTION         := OP;                               {Btrieve operation}
        USER_KEY_ADDR.OFFSET  := sizeof(BTR_PARMS) + sizeof(STAT)
                                 + 128 - 1;                        {key buffer address}
        USER_KEY_ADDR.SEGMENT := Para;
        USER_KEY_LENGTH       := 255;                              {key buffer length}
        USER_KEY_NUMBER       := KEY;                              {key number}
        USER_STAT_ADDR.OFFSET := sizeof(BTR_PARMS) - 1;            {return status address}
        USER_STAT_ADDR.SEGMENT:= Para;
        XFACE_ID              := VAR_ID;                           {language interface id}
      end;
    Move(POS,Ptr(Sel,sizeof(BTR_PARMS)+sizeof(Stat)-1)^,128);      { Move POS Blk             }
    Move(KBUF,Ptr(Sel,sizeof(BTR_PARMS)+sizeof(Stat)+128-1)^,255); { Move Key Buffer          }
    Move(DATA,Ptr(Sel,sizeof(BTR_PARMS)+sizeof(Stat)+128+255-1)^,DATALEN); { Move Data Buffer}
    With DPMIRegs do
      begin
        EBX   := Regs.BX;
        EAX   := Regs.AX;
        Flags := 0;
        EDX   := 0;
        DS    := Para;
        EDI   := 0;
        ESI   := 0;
        EBP   := 0;
        ECX   := 0;
        ES    := 0;
        FS    := 0;
        GS    := 0;
        IP    := 0;
        CS    := 0;
        SP    := 0;
        SS    := 0;
      end;
    if DPMI_RMode_Intr(IntNo, 0, 0, DPMIRegs) then {Invoke Real Mode Interrupt Using DPMI }
        begin
          Move(Ptr(Sel,sizeof(BTR_PARMS)-1)^,STAT,sizeof(STAT));         { Get Status          }
          Move(Ptr(Sel,sizeof(BTR_PARMS)+sizeof(Stat)-1)^,POS,128);      { Restore POS Blk     }
          Move(Ptr(Sel,sizeof(BTR_PARMS)+sizeof(Stat)+128-1)^,KBUF,255); { Restore Key Buffer  }
          Move(Ptr(Sel,sizeof(BTR_PARMS)+sizeof(Stat)+128+255-1)^,DATA,DATALEN); {Restore Data Buffer}
        end
      else STAT := 9996; {Real Mode Interrupt No Good}
    If not DosFreeRealSegOK(Sel) then STAT := 9997;  {Could Not Free Up Low Memory }
  end;
{$ENDIF}

begin
  {$IFDEF WINDOWS}
  if (GetWinFlags and wf_PMode) > 0 then {Protected Mode Test}
      begin
        If not DPMIPresent then {Test For DPMI}
          begin
            BTRV := 9999;
            exit;
          end;
        DPMI_RMode_Get_Intr_Vector(BTR_INT,Paragraph,Offset);
        If Offset <> $33 then {make sure Btrieve is installed}
          begin
            BTRV := 20;
            exit;
          end;
      end
    else if MemW[$0000:$01ec] <> $0033 then {make sure Btrieve is installed}
      begin
        BTRV := 20;
        exit;
      end;
  {$ENDIF}
  {$IFNDEF WINDOWS}
  if MemW[$0000:$01ec] <> $0033 then {make sure Btrieve is installed - DOS}
    begin
      BTRV := 20;
      exit;
    end;
  {$ENDIF}
  if (not VSet) then   {if we haven't checked for Multi-User version}
    begin
      VSet := true;
      STAT := BTRV (26, POS, tbuf, i, KBUF, KEY);
      MULTI := tbuf[5] = 'M';     {is Multi-User version installed}
    end;
  {make normal btrieve call}
  with XDATA do
    begin
      USER_BUF_ADDR.SEGMENT := SEG (DATA);
      USER_BUF_ADDR.OFFSET := OFS (DATA); {set data buffer address}
      USER_BUF_LEN := DATALEN;
      USER_FCB_ADDR.SEGMENT := SEG (POS);
      USER_FCB_ADDR.OFFSET := OFS (POS);          {set FCB address}
      USER_CUR_ADDR.SEGMENT := USER_FCB_ADDR.SEGMENT; {set cur seg}
      USER_CUR_ADDR.OFFSET := USER_FCB_ADDR.OFFSET+38;{set cur ofs}
      USER_FUNCTION := OP;             {set Btrieve operation code}
      USER_KEY_ADDR.SEGMENT := SEG (KBUF);
      USER_KEY_ADDR.OFFSET := OFS (KBUF);  {set key buffer address}
      USER_KEY_LENGTH := 255;             {assume its large enough}
      USER_KEY_NUMBER := KEY;                      {set key number}
      USER_STAT_ADDR.SEGMENT := SEG (STAT);
      USER_STAT_ADDR.OFFSET := OFS (STAT);     {set status address}
      XFACE_ID := VAR_ID;                         {set lamguage id}
    end;

  if (MULTI) then
    begin
      REGS.AX := 1;
      while (REGS.AX <> 0) do          {wait for bmulti to be free}
        begin
          REGS.AX := 0;
          INTR (BTR_INT, REGS);
          if (REGS.AX <> 0) then
          for i := 0 to 1000 do
          i := i + 1;
        end;
    end;
  REGS.BX := ProcId;
  REGS.AX := 1;
  if (REGS.BX <> 0) then
  REGS.AX := 2;
  Typ := REGS.AX;
  if ((not MULTI) or (OP < 100) or (OP >= 200)) then
      begin                         {not a waited lock call to BMulti}
        REGS.DX := OFS (XDATA);
        REGS.DS := SEG (XDATA);
        INTR (BTR_INT, REGS);
      end
    else
      begin
        repeat
          REGS.BX := ProcId;   {waited lock calls to BMulti must be}
          REGS.AX := 1;        {handled separately                 }
          if (REGS.BX <> 0) then REGS.AX := 2;
          XDATA.USER_BUF_LEN := DATALEN;
          XDATA.USER_FUNCTION := OP + 100;
          REGS.DX := OFS (XDATA);
          REGS.DS := SEG (XDATA);
          INTR (BTR_INT, REGS);
          if ((STAT = 84) or (STAT = 85)) then
          for i := 0 to 1000 do i := i + 1;
        until ((STAT <> 84) and (STAT <> 85)); {wait for no conflict}
      end;
  if (Typ = 1) then ProcId := REGS.BX;
  {             retrieve return data length             }
  DATALEN := XDATA.USER_BUF_LEN;
  BTRV := STAT;
end;

begin
end.
